@@ -47,6 +47,7 @@ struct _PhotosBaseItemPrivate
   GeglNode *graph;
   GeglNode *node;
   GeglRectangle bbox;
+  GMutex mutex_create_thumbnail;
   GMutex mutex;
   PhotosCollectionIconWatcher *watcher;
   TrackerSparqlCursor *cursor;
@@ -54,7 +55,6 @@ struct _PhotosBaseItemPrivate
   gboolean failed_thumbnailing;
   gboolean favorite;
   gboolean thumbnailed;
-  gboolean tried_thumbnailing;
   const gchar *thumb_path;
   gchar *author;
   gchar *default_app_name;
@@ -76,7 +76,6 @@ enum
   PROP_CURSOR,
   PROP_FAILED_THUMBNAILING,
   PROP_ID,
-  PROP_TRIED_THUMBNAILING
 };
 
 enum
@@ -200,35 +199,69 @@ photos_base_item_check_effects_and_update_info (PhotosBaseItem *self)
 }
 
 
-static GeglNode *
-photos_base_item_default_load (PhotosBaseItem *self, GCancellable *cancellable, GError **error)
+static void
+photos_base_item_create_thumbnail_in_thread_func (GSimpleAsyncResult *simple,
+                                                  GObject *object,
+                                                  GCancellable *cancellable)
 {
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (object);
   PhotosBaseItemPrivate *priv = self->priv;
-  GeglNode *ret_val = NULL;
-  GFile *file = NULL;
-  gchar *path = NULL;
+  GError *error;
+  gboolean op_res;
 
-  if (priv->graph == NULL)
-    {
-      file = g_file_new_for_uri (priv->uri);
-      path = g_file_get_path (file);
-      if (path == NULL)
-        goto out;
+  g_mutex_lock (&priv->mutex_create_thumbnail);
 
-      priv->graph = gegl_node_new ();
-      priv->node = gegl_node_new_child (priv->graph,
-                                        "operation", "gegl:load",
-                                        "path", path,
-                                        NULL);
-    }
+  error = NULL;
+  op_res = PHOTOS_BASE_ITEM_GET_CLASS (self)->create_thumbnail (self, cancellable, &error);
+  if (error != NULL)
+    g_simple_async_result_take_error (simple, error);
 
-  gegl_node_process (priv->node);
-  priv->bbox = gegl_node_get_bounding_box (priv->node);
-  ret_val = g_object_ref (priv->node);
+  g_simple_async_result_set_op_res_gboolean (simple, op_res);
+
+  g_mutex_unlock (&priv->mutex_create_thumbnail);
+}
+
+
+static void
+photos_base_item_create_thumbnail_async (PhotosBaseItem *self,
+                                         GCancellable *cancellable,
+                                         GAsyncReadyCallback callback,
+                                         gpointer user_data)
+{
+  GSimpleAsyncResult *simple;
+
+  simple = g_simple_async_result_new (G_OBJECT (self),
+                                      callback,
+                                      user_data,
+                                      photos_base_item_create_thumbnail_async);
+  g_simple_async_result_set_check_cancellable (simple, cancellable);
+
+  g_simple_async_result_run_in_thread (simple,
+                                       photos_base_item_create_thumbnail_in_thread_func,
+                                       G_PRIORITY_DEFAULT,
+                                       cancellable);
+  g_object_unref (simple);
+}
+
+
+static gboolean
+photos_base_item_create_thumbnail_finish (PhotosBaseItem *self, GAsyncResult *res, GError **error)
+{
+  GSimpleAsyncResult *simple = G_SIMPLE_ASYNC_RESULT (res);
+  gboolean ret_val = FALSE;
+
+  g_return_val_if_fail (g_simple_async_result_is_valid (res,
+                                                        G_OBJECT (self),
+                                                        photos_base_item_create_thumbnail_async),
+                        NULL);
+  g_return_val_if_fail (error == NULL || *error == NULL, NULL);
+
+  if (g_simple_async_result_propagate_error (simple, error))
+    goto out;
+
+  ret_val = g_simple_async_result_get_op_res_gboolean (simple);
 
  out:
-  g_free (path);
-  g_clear_object (&file);
   return ret_val;
 }
 
@@ -388,17 +421,20 @@ photos_base_item_thumbnail_path_info (GObject *source_object, GAsyncResult *res,
 
 
 static void
-photos_base_item_queue_thumbnail_job (GObject *source_object, GAsyncResult *res, gpointer user_data)
+photos_base_item_create_thumbnail_cb (GObject *source_object, GAsyncResult *res, gpointer user_data)
 {
-  PhotosBaseItem *self = PHOTOS_BASE_ITEM (user_data);
+  PhotosBaseItem *self = PHOTOS_BASE_ITEM (source_object);
   PhotosBaseItemPrivate *priv = self->priv;
-  GFile *file = G_FILE (source_object);
-  gboolean thumbnailed;
+  GError *error;
+  GFile *file = G_FILE (user_data);
 
-  thumbnailed = photos_utils_queue_thumbnail_job_for_file_finish (res);
-  if (!thumbnailed)
+  error = NULL;
+  photos_base_item_create_thumbnail_finish (self, res, &error);
+  if (error != NULL)
     {
       priv->failed_thumbnailing = TRUE;
+      g_warning ("Unable to create thumbnail: %s", error->message);
+      g_error_free (error);
       goto out;
     }
 
@@ -411,6 +447,7 @@ photos_base_item_queue_thumbnail_job (GObject *source_object, GAsyncResult *res,
                            g_object_ref (self));
 
  out:
+  g_object_unref (file);
   g_object_unref (self);
 }
 
@@ -423,7 +460,6 @@ photos_base_item_file_query_info (GObject *source_object, GAsyncResult *res, gpo
   GError *error = NULL;
   GFile *file = G_FILE (source_object);
   GFileInfo *info;
-  gboolean have_new_icon = FALSE;
 
   info = g_file_query_info_finish (file, res, &error);
   if (error != NULL)
@@ -440,9 +476,10 @@ photos_base_item_file_query_info (GObject *source_object, GAsyncResult *res, gpo
   else
     {
       priv->thumbnailed = FALSE;
-      photos_utils_queue_thumbnail_job_for_file_async (file,
-                                                       photos_base_item_queue_thumbnail_job,
-                                                       g_object_ref (self));
+      photos_base_item_create_thumbnail_async (self,
+                                               NULL,
+                                               photos_base_item_create_thumbnail_cb,
+                                               g_object_ref (file));
     }
 
  out:
@@ -450,6 +487,36 @@ photos_base_item_file_query_info (GObject *source_object, GAsyncResult *res, gpo
 }
 
 
+static GeglNode *
+photos_base_item_load (PhotosBaseItem *self, GCancellable *cancellable, GError **error)
+{
+  PhotosBaseItemPrivate *priv = self->priv;
+  GeglNode *ret_val = NULL;
+  gchar *path = NULL;
+
+  if (priv->graph == NULL)
+    {
+      path = PHOTOS_BASE_ITEM_GET_CLASS (self)->download(self, cancellable, error);
+      if (path == NULL)
+        goto out;
+
+      priv->graph = gegl_node_new ();
+      priv->node = gegl_node_new_child (priv->graph,
+                                        "operation", "gegl:load",
+                                        "path", path,
+                                        NULL);
+    }
+
+  gegl_node_process (priv->node);
+  priv->bbox = gegl_node_get_bounding_box (priv->node);
+  ret_val = g_object_ref (priv->node);
+
+ out:
+  g_free (path);
+  return ret_val;
+}
+
+
 static void
 photos_base_item_load_in_thread_func (GSimpleAsyncResult *simple, GObject *object, GCancellable *cancellable)
 {
@@ -460,7 +527,7 @@ photos_base_item_load_in_thread_func (GSimpleAsyncResult *simple, GObject *objec
 
   g_mutex_lock (&priv->mutex);
 
-  node = PHOTOS_BASE_ITEM_GET_CLASS (self)->load (self, cancellable, &error);
+  node = photos_base_item_load (self, cancellable, &error);
   if (error != NULL)
     g_simple_async_result_take_error (simple, error);
 
@@ -524,9 +591,6 @@ photos_base_item_refresh_icon (PhotosBaseItem *self)
   if (priv->failed_thumbnailing)
     return;
 
-  if (!priv->tried_thumbnailing)
-    priv->tried_thumbnailing = TRUE;
-
   file = g_file_new_for_uri (priv->uri);
   g_file_query_info_async (file,
                            G_FILE_ATTRIBUTE_THUMBNAIL_PATH,
@@ -680,6 +744,7 @@ photos_base_item_finalize (GObject *object)
   g_free (priv->type_description);
   g_free (priv->uri);
 
+  g_mutex_clear (&priv->mutex_create_thumbnail);
   g_mutex_clear (&priv->mutex);
 
   G_OBJECT_CLASS (photos_base_item_parent_class)->finalize (object);
@@ -720,10 +785,6 @@ photos_base_item_set_property (GObject *object, guint prop_id, const GValue *val
       priv->failed_thumbnailing = g_value_get_boolean (value);
       break;
 
-    case PROP_TRIED_THUMBNAILING:
-      priv->tried_thumbnailing = g_value_get_boolean (value);
-      break;
-
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -739,6 +800,7 @@ photos_base_item_init (PhotosBaseItem *self)
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_BASE_ITEM, PhotosBaseItemPrivate);
   priv = self->priv;
 
+  g_mutex_init (&priv->mutex_create_thumbnail);
   g_mutex_init (&priv->mutex);
 }
 
@@ -753,7 +815,6 @@ photos_base_item_class_init (PhotosBaseItemClass *class)
   object_class->finalize = photos_base_item_finalize;
   object_class->get_property = photos_base_item_get_property;
   object_class->set_property = photos_base_item_set_property;
-  class->load = photos_base_item_default_load;
   class->set_favorite = photos_base_item_default_set_favorite;
   class->update_type_description = photos_base_item_update_type_description;
 
@@ -781,14 +842,6 @@ photos_base_item_class_init (PhotosBaseItemClass *class)
                                                         "",
                                                         G_PARAM_READABLE));
 
-  g_object_class_install_property (object_class,
-                                   PROP_TRIED_THUMBNAILING,
-                                   g_param_spec_boolean ("tried-thumbnailing",
-                                                         "Thumbnailing attempted",
-                                                         "An attempt was made to create a thumbnail",
-                                                         FALSE,
-                                                         G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE));
-
   signals[INFO_UPDATED] = g_signal_new ("info-updated",
                                         G_TYPE_FROM_CLASS (class),
                                         G_SIGNAL_RUN_LAST,
