@@ -68,14 +68,15 @@ struct _PhotosEmbedPrivate
   PhotosTrackerChangeMonitor *monitor;
   PhotosTrackerController *trk_ovrvw_cntrlr;
   guint load_show_id;
-  gulong no_results_change_id;
-  gulong notify_visible_child_id;
 };
 
 
 G_DEFINE_TYPE (PhotosEmbed, photos_embed, GTK_TYPE_BOX);
 
 
+static void photos_embed_changes_pending (PhotosEmbed *self, GHashTable *changes);
+
+
 static void
 photos_embed_clear_load_timer (PhotosEmbed *self)
 {
@@ -195,12 +196,7 @@ photos_embed_hide_no_results_page (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv = self->priv;
 
-  if (priv->no_results_change_id != 0)
-    {
-      g_signal_handler_disconnect (priv->monitor, priv->no_results_change_id);
-      priv->no_results_change_id = 0;
-    }
-
+  g_signal_handlers_disconnect_by_func (priv->monitor, photos_embed_changes_pending, self);
   photos_embed_restore_last_page (self);
 }
 
@@ -224,10 +220,11 @@ photos_embed_count_changed (PhotosEmbed *self, gint count)
 
   if (count == 0)
     {
-      priv->no_results_change_id = g_signal_connect_swapped (priv->monitor,
-                                                             "changes-pending",
-                                                             G_CALLBACK (photos_embed_changes_pending),
-                                                             self);
+      g_signal_connect_object (priv->monitor,
+                               "changes-pending",
+                               G_CALLBACK (photos_embed_changes_pending),
+                               self,
+                               G_CONNECT_SWAPPED);
       gd_stack_set_visible_child_name (GD_STACK (priv->stack), "no-results");
     }
   else
@@ -246,7 +243,6 @@ photos_embed_notify_visible_child (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv = self->priv;
   GtkWidget *visible_child;
-  PhotosWindowMode mode;
 
   visible_child = gd_stack_get_visible_child (GD_STACK (priv->stack));
   if (visible_child == priv->overview)
@@ -326,7 +322,7 @@ photos_embed_prepare_for_preview (PhotosEmbed *self)
 }
 
 
-void
+static void
 photos_embed_set_error (PhotosEmbed *self, const gchar *primary, const gchar *secondary)
 {
   PhotosEmbedPrivate *priv = self->priv;
@@ -343,7 +339,7 @@ photos_embed_query_error (PhotosEmbed *self, const gchar *primary, const gchar *
 }
 
 
-void
+static void
 photos_embed_query_status_changed (PhotosTrackerController *trk_cntrlr, gboolean querying, gpointer user_data)
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
@@ -368,7 +364,6 @@ photos_embed_window_mode_changed (PhotosModeController *mode_cntrlr,
                                   gpointer user_data)
 {
   PhotosEmbed *self = PHOTOS_EMBED (user_data);
-  PhotosEmbedPrivate *priv = self->priv;
 
   if (mode == PHOTOS_WINDOW_MODE_COLLECTIONS)
     photos_embed_prepare_for_collections (self);
@@ -387,18 +382,6 @@ photos_embed_dispose (GObject *object)
   PhotosEmbed *self = PHOTOS_EMBED (object);
   PhotosEmbedPrivate *priv = self->priv;
 
-  if (priv->no_results_change_id != 0)
-    {
-      g_signal_handler_disconnect (priv->monitor, priv->no_results_change_id);
-      priv->no_results_change_id = 0;
-    }
-
-  if (priv->notify_visible_child_id != 0)
-    {
-      g_signal_handler_disconnect (priv->stack, priv->notify_visible_child_id);
-      priv->notify_visible_child_id = 0;
-    }
-
   g_clear_object (&priv->ntfctn_mngr);
   g_clear_object (&priv->loader_cancellable);
   g_clear_object (&priv->indexing_ntfctn);
@@ -408,6 +391,17 @@ photos_embed_dispose (GObject *object)
   g_clear_object (&priv->monitor);
   g_clear_object (&priv->trk_ovrvw_cntrlr);
 
+  /* GdStack triggers notify::visible-child during dispose and this means that
+   * we have to explicitly disconnect the signal handler before calling up to
+   * the parent implementation, or photos_embed_notify_visible_child() will
+   * get called while we're in a inconsistent state
+   */
+  if (priv->stack != NULL)
+    {
+      g_signal_handlers_disconnect_by_func (priv->stack, photos_embed_notify_visible_child, self);
+      priv->stack = NULL;
+    }
+
   G_OBJECT_CLASS (photos_embed_parent_class)->dispose (object);
 }
 
@@ -416,7 +410,6 @@ photos_embed_init (PhotosEmbed *self)
 {
   PhotosEmbedPrivate *priv;
   gboolean querying;
-  GtkStyleContext *context;
 
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self, PHOTOS_TYPE_EMBED, PhotosEmbedPrivate);
   priv = self->priv;
@@ -464,17 +457,15 @@ photos_embed_init (PhotosEmbed *self)
   priv->no_results = photos_empty_results_box_new ();
   gd_stack_add_named (GD_STACK (priv->stack), priv->no_results, "no-results");
 
-  /* TODO: SearchBar.Dropdown,
-   *       ...
+  /* TODO: SearchBar.Dropdown, …
    */
 
   priv->selection_toolbar = photos_selection_toolbar_new ();
   gtk_overlay_add_overlay (GTK_OVERLAY (priv->stack_overlay), priv->selection_toolbar);
 
-  priv->notify_visible_child_id = g_signal_connect_swapped (priv->stack,
-                                                            "notify::visible-child",
-                                                            G_CALLBACK (photos_embed_notify_visible_child),
-                                                            self);
+  g_signal_connect_object (priv->stack, "notify::visible-child",
+                           G_CALLBACK (photos_embed_notify_visible_child),
+                           self, G_CONNECT_SWAPPED);
 
   priv->mode_cntrlr = photos_mode_controller_new ();
   g_signal_connect (priv->mode_cntrlr,
