@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,16 +29,48 @@
 #include <glib.h>
 
 #include "photos-local-item.h"
+#include "photos-utils.h"
 
 
 G_DEFINE_TYPE (PhotosLocalItem, photos_local_item, PHOTOS_TYPE_BASE_ITEM);
 
 
+static gboolean
+photos_local_item_create_thumbnail (PhotosBaseItem *item, GCancellable *cancellable, GError **error)
+{
+  GFile *file;
+  gboolean ret_val;
+  const gchar *uri;
+
+  uri = photos_base_item_get_uri (item);
+  file = g_file_new_for_uri (uri);
+  ret_val = photos_utils_create_thumbnail (file, cancellable, error);
+
+  g_object_unref (file);
+  return ret_val;
+}
+
+
+static gchar *
+photos_local_item_download (PhotosBaseItem *item, GCancellable *cancellable, GError **error)
+{
+  GFile *file = NULL;
+  const gchar *uri;
+  gchar *path = NULL;
+
+  uri = photos_base_item_get_uri (item);
+  file = g_file_new_for_uri (uri);
+  path = g_file_get_path (file);
+
+  g_object_unref (file);
+  return path;
+}
+
+
 static void
 photos_local_item_constructed (GObject *object)
 {
   PhotosLocalItem *self = PHOTOS_LOCAL_ITEM (object);
-  PhotosLocalItemPrivate *priv = self->priv;
   GAppInfo *default_app = NULL;
   const gchar *default_app_name;
   const gchar *mime_type;
@@ -70,8 +102,11 @@ static void
 photos_local_item_class_init (PhotosLocalItemClass *class)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (class);
+  PhotosBaseItemClass *base_item_class = PHOTOS_BASE_ITEM_CLASS (class);
 
   object_class->constructed= photos_local_item_constructed;
+  base_item_class->create_thumbnail = photos_local_item_create_thumbnail;
+  base_item_class->download = photos_local_item_download;
 }
 
 
@@ -81,6 +116,5 @@ photos_local_item_new (TrackerSparqlCursor *cursor)
   return g_object_new (PHOTOS_TYPE_LOCAL_ITEM,
                        "cursor", cursor,
                        "failed-thumbnailing", FALSE,
-                       "tried-thumbnailing", FALSE,
                        NULL);
 }
