@@ -32,10 +32,12 @@
 
 #include "photos-application.h"
 #include "photos-collection-manager.h"
+#include "photos-dlna-renderers-manager.h"
 #include "photos-header-bar.h"
 #include "photos-item-manager.h"
 #include "photos-main-toolbar.h"
 #include "photos-mode-controller.h"
+#include "photos-remote-display-manager.h"
 #include "photos-selection-controller.h"
 #include "photos-source-manager.h"
 
@@ -44,19 +46,16 @@ struct _PhotosMainToolbarPrivate
 {
   GSimpleAction *gear_menu;
   GtkWidget *coll_back_button;
+  GtkWidget *remote_display_button;
   GtkWidget *selection_menu;
   GtkWidget *toolbar;
   PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
   PhotosModeController *mode_cntrlr;
+  PhotosRemoteDisplayManager *remote_mngr;
   PhotosSelectionController *sel_cntrlr;
   PhotosWindowMode old_mode;
-  gulong collection_id;
-  gulong search_source_id;
-  gulong selection_changed_id;
-  gulong selection_mode_id;
-  gulong window_mode_id;
 };
 
 
@@ -83,7 +82,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
       if (!selection_mode)
         {
           if (active_collection != NULL)
-            primary = g_strdup (photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)));
+            primary = g_markup_printf_escaped ("%s", photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)));
           else
             {
             }
@@ -103,14 +102,13 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
 
           if (active_collection != NULL)
             {
-              primary = g_strdup_printf ("<b>%s</b> (%s)",
-                                         photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)),
-                                         label);
+              primary =  g_markup_printf_escaped ("<b>%s</b> (%s)",
+                                                  photos_base_item_get_name (PHOTOS_BASE_ITEM (active_collection)),
+                                                  label);
             }
           else
             {
-              primary = label;
-              label = NULL;
+              primary = g_markup_printf_escaped ("%s", label);
             }
 
           g_free (label);
@@ -121,7 +119,7 @@ photos_main_toolbar_set_toolbar_title (PhotosMainToolbar *self)
       GObject *item;
 
       item = photos_base_manager_get_active_object (priv->item_mngr);
-      primary = g_strdup (photos_base_item_get_name (PHOTOS_BASE_ITEM (item)));
+      primary = g_markup_printf_escaped ("%s", photos_base_item_get_name (PHOTOS_BASE_ITEM (item)));
     }
 
   if (selection_mode)
@@ -207,6 +205,48 @@ photos_main_toolbar_select_button_clicked (GtkButton *button, gpointer user_data
 }
 
 
+static void
+photos_main_toolbar_remote_display_button_clicked (GtkButton *button,
+                                                   gpointer user_data)
+{
+  PhotosMainToolbar *self = PHOTOS_MAIN_TOOLBAR (user_data);
+  PhotosMainToolbarPrivate *priv = self->priv;
+
+  photos_remote_display_manager_stop (priv->remote_mngr);
+}
+
+
+static void
+photos_main_toolbar_add_remote_display_button (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  PhotosDlnaRenderer *renderer;
+  GtkLabel *label;
+  gchar *text;
+  const gchar *name;
+
+  if (priv->remote_display_button != NULL)
+    gtk_widget_destroy (priv->remote_display_button);
+
+  renderer = photos_remote_display_manager_get_renderer (priv->remote_mngr);
+  name = photos_dlna_renderer_get_friendly_name (renderer);
+  text = g_markup_printf_escaped ("Displaying on <b>%s</b>", name);
+
+  priv->remote_display_button = gd_header_simple_button_new ();
+  gd_header_button_set_label (GD_HEADER_BUTTON (priv->remote_display_button), text);
+  gd_header_button_set_use_markup (GD_HEADER_BUTTON (priv->remote_display_button), TRUE);
+  label = GTK_LABEL (gtk_bin_get_child (GTK_BIN (priv->remote_display_button)));
+  gtk_label_set_ellipsize (label, PANGO_ELLIPSIZE_MIDDLE);
+  gtk_widget_set_margin_right (priv->remote_display_button, 12);
+  gd_header_bar_pack_start (GD_HEADER_BAR (priv->toolbar), priv->remote_display_button);
+  gtk_widget_show_all (priv->remote_display_button);
+
+  g_signal_connect (priv->remote_display_button, "clicked",
+                    G_CALLBACK (photos_main_toolbar_remote_display_button_clicked), self);
+  g_free (text);
+}
+
+
 static void
 photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
 {
@@ -219,10 +259,11 @@ photos_main_toolbar_add_selection_button (PhotosMainToolbar *self)
   gd_header_bar_pack_end (GD_HEADER_BAR (priv->toolbar), selection_button);
   g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_select_button_clicked), self);
 
-  priv->collection_id = g_signal_connect (priv->col_mngr,
-                                          "active-changed",
-                                          G_CALLBACK (photos_main_toolbar_active_changed),
-                                          self);
+  g_signal_connect_object (priv->col_mngr,
+                           "active-changed",
+                           G_CALLBACK (photos_main_toolbar_active_changed),
+                           self,
+                           0);
 }
 
 
@@ -248,17 +289,17 @@ photos_main_toolbar_clear_state_data (PhotosMainToolbar *self)
       priv->coll_back_button = NULL;
     }
 
-  if (priv->collection_id != 0)
+  if (priv->remote_display_button != NULL)
     {
-      g_signal_handler_disconnect (priv->col_mngr, priv->collection_id);
-      priv->collection_id = 0;
+      gtk_widget_destroy (priv->remote_display_button);
+      priv->remote_display_button = NULL;
     }
 
-  if (priv->selection_changed_id != 0)
-    {
-      g_signal_handler_disconnect (priv->sel_cntrlr, priv->selection_changed_id);
-      priv->selection_changed_id = 0;
-    }
+  if (priv->col_mngr != NULL)
+    g_signal_handlers_disconnect_by_func (priv->col_mngr, photos_main_toolbar_active_changed, self);
+
+  if (priv->sel_cntrlr != NULL)
+    g_signal_handlers_disconnect_by_func (priv->sel_cntrlr, photos_main_toolbar_set_toolbar_title, self);
 }
 
 
@@ -331,6 +372,24 @@ photos_main_toolbar_populate_for_collections (PhotosMainToolbar *self)
 }
 
 
+static void
+photos_main_toolbar_update_remote_display_button (PhotosMainToolbar *self)
+{
+  PhotosMainToolbarPrivate *priv = self->priv;
+  PhotosWindowMode window_mode;
+  gboolean selection_mode, active;
+
+  selection_mode = photos_selection_controller_get_selection_mode (priv->sel_cntrlr);
+  window_mode = photos_mode_controller_get_window_mode (priv->mode_cntrlr);
+  active = photos_remote_display_manager_is_active (priv->remote_mngr);
+
+  if (active && !selection_mode && window_mode != PHOTOS_WINDOW_MODE_PREVIEW)
+    photos_main_toolbar_add_remote_display_button (self);
+  else
+    g_clear_pointer (&priv->remote_display_button, gtk_widget_destroy);
+}
+
+
 static void
 photos_main_toolbar_populate_for_favorites (PhotosMainToolbar *self)
 {
@@ -366,6 +425,9 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   GMenu *preview_menu;
   GtkWidget *back_button;
   GtkWidget *menu_button;
+  GtkApplication *app;
+  gboolean remote_display_available;
+  GAction *remote_display_action;
 
   photos_header_bar_set_mode (PHOTOS_HEADER_BAR (priv->toolbar), PHOTOS_HEADER_BAR_MODE_STANDALONE);
 
@@ -380,6 +442,13 @@ photos_main_toolbar_populate_for_preview (PhotosMainToolbar *self)
   gd_header_bar_pack_end (GD_HEADER_BAR (priv->toolbar), menu_button);
 
   g_simple_action_set_enabled (priv->gear_menu, TRUE);
+
+  /* Disable the remote-display-current action if the dLeyna services are not
+   * available */
+  app = photos_application_new ();
+  remote_display_action = g_action_map_lookup_action (G_ACTION_MAP (app), "remote-display-current");
+  remote_display_available = photos_dlna_renderers_manager_is_available ();
+  g_simple_action_set_enabled (G_SIMPLE_ACTION (remote_display_action), remote_display_available);
 }
 
 
@@ -399,10 +468,11 @@ photos_main_toolbar_populate_for_selection_mode (PhotosMainToolbar *self)
   gtk_style_context_add_class (context, "suggested-action");
   g_signal_connect (selection_button, "clicked", G_CALLBACK (photos_main_toolbar_done_button_clicked), self);
 
-  priv->selection_changed_id = g_signal_connect_swapped (priv->sel_cntrlr,
-                                                         "selection-changed",
-                                                         G_CALLBACK (photos_main_toolbar_set_toolbar_title),
-                                                         self);
+  g_signal_connect_object (priv->sel_cntrlr,
+                           "selection-changed",
+                           G_CALLBACK (photos_main_toolbar_set_toolbar_title),
+                           self,
+                           G_CONNECT_SWAPPED);
 }
 
 
@@ -428,6 +498,8 @@ photos_main_toolbar_reset_toolbar_mode (PhotosMainToolbar *self)
   else if (window_mode == PHOTOS_WINDOW_MODE_PREVIEW)
     photos_main_toolbar_populate_for_preview (self);
 
+  photos_main_toolbar_update_remote_display_button (self);
+
   photos_main_toolbar_set_toolbar_title (self);
   gtk_widget_show_all (priv->toolbar);
 }
@@ -449,34 +521,40 @@ photos_main_toolbar_dispose (GObject *object)
 
   photos_main_toolbar_clear_state_data (self);
 
-  if (priv->window_mode_id != 0)
-    {
-      g_signal_handler_disconnect (priv->mode_cntrlr, priv->window_mode_id);
-      priv->window_mode_id = 0;
-    }
-
-  if (priv->selection_mode_id != 0)
-    {
-      g_signal_handler_disconnect (priv->sel_cntrlr, priv->selection_mode_id);
-      priv->selection_mode_id = 0;
-    }
-
-  if (priv->search_source_id != 0)
-    {
-      g_signal_handler_disconnect (priv->src_mngr, priv->search_source_id);
-      priv->search_source_id = 0;
-    }
-
   g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
   g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->remote_mngr);
   g_clear_object (&priv->sel_cntrlr);
 
   G_OBJECT_CLASS (photos_main_toolbar_parent_class)->dispose (object);
 }
 
 
+static void
+photos_main_toolbar_share_changed_cb (PhotosMainToolbar          *self,
+                                      PhotosDlnaRenderer         *renderer,
+                                      PhotosBaseItem             *item,
+                                      PhotosRemoteDisplayManager *remote_mngr)
+{
+  photos_main_toolbar_update_remote_display_button (self);
+}
+
+
+static void
+photos_main_toolbar_share_error_cb (PhotosMainToolbar          *self,
+                                    PhotosDlnaRenderer         *renderer,
+                                    PhotosBaseItem             *item,
+                                    GError                     *error,
+                                    PhotosRemoteDisplayManager *remote_mngr)
+{
+  photos_main_toolbar_update_remote_display_button (self);
+
+  g_warning ("Error sharing item with remote display: %s", error->message);
+}
+
+
 static void
 photos_main_toolbar_init (PhotosMainToolbar *self)
 {
@@ -515,23 +593,43 @@ photos_main_toolbar_init (PhotosMainToolbar *self)
   priv->item_mngr = photos_item_manager_new ();
 
   priv->src_mngr = photos_source_manager_new ();
-  priv->search_source_id = g_signal_connect_swapped (priv->src_mngr,
-                                                     "active-changed",
-                                                     G_CALLBACK (photos_main_toolbar_set_toolbar_title),
-                                                     self);
+  g_signal_connect_object (priv->src_mngr,
+                           "active-changed",
+                           G_CALLBACK (photos_main_toolbar_set_toolbar_title),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   priv->mode_cntrlr = photos_mode_controller_new ();
   priv->old_mode = PHOTOS_WINDOW_MODE_NONE;
-  priv->window_mode_id = g_signal_connect_swapped (priv->mode_cntrlr,
-                                                   "window-mode-changed",
-                                                   G_CALLBACK (photos_main_toolbar_window_mode_changed),
-                                                   self);
+  g_signal_connect_object (priv->mode_cntrlr,
+                           "window-mode-changed",
+                           G_CALLBACK (photos_main_toolbar_window_mode_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   priv->sel_cntrlr = photos_selection_controller_new ();
-  priv->selection_mode_id = g_signal_connect_swapped (priv->sel_cntrlr,
-                                                      "selection-mode-changed",
-                                                      G_CALLBACK (photos_main_toolbar_reset_toolbar_mode),
-                                                      self);
+  g_signal_connect_object (priv->sel_cntrlr,
+                           "selection-mode-changed",
+                           G_CALLBACK (photos_main_toolbar_reset_toolbar_mode),
+                           self,
+                           G_CONNECT_SWAPPED);
+
+  priv->remote_mngr = photos_remote_display_manager_dup_singleton ();
+  g_signal_connect_object (priv->remote_mngr,
+                           "share-began",
+                           G_CALLBACK (photos_main_toolbar_share_changed_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->remote_mngr,
+                           "share-ended",
+                           G_CALLBACK (photos_main_toolbar_share_changed_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (priv->remote_mngr,
+                           "share-error",
+                           G_CALLBACK (photos_main_toolbar_share_error_cb),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   photos_main_toolbar_reset_toolbar_mode (self);
 }
