@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -31,16 +31,26 @@
 
 G_BEGIN_DECLS
 
-PhotosQuery  *photos_query_builder_count_favorites_query (void);
+PhotosQuery  *photos_query_builder_create_collection_query (const gchar *name);
 
-PhotosQuery  *photos_query_builder_count_query         (void);
+PhotosQuery  *photos_query_builder_collection_icon_query (const gchar *resource);
 
-PhotosQuery  *photos_query_builder_global_favorites_query (void);
+PhotosQuery  *photos_query_builder_count_query (gint flags);
 
-PhotosQuery  *photos_query_builder_global_query        (void);
+PhotosQuery  *photos_query_builder_delete_resource_query (const gchar *resource);
+
+PhotosQuery  *photos_query_builder_fetch_collections_query (const gchar *resource);
+
+PhotosQuery  *photos_query_builder_global_query        (gint flags);
+
+PhotosQuery  *photos_query_builder_set_collection_query (const gchar *item_urn,
+                                                         const gchar *collection_urn,
+                                                         gboolean setting);
 
 PhotosQuery  *photos_query_builder_single_query        (gint flags, const gchar *resource);
 
+PhotosQuery  *photos_query_builder_update_mtime_query (const gchar *resource);
+
 gchar        *photos_query_builder_filter_local        (void);
 
 G_END_DECLS
