@@ -105,7 +105,6 @@ static void
 photos_selection_toolbar_collection_clicked (GtkButton *button, gpointer user_data)
 {
   PhotosSelectionToolbar *self = PHOTOS_SELECTION_TOOLBAR (user_data);
-  PhotosSelectionToolbarPrivate *priv = self->priv;
   GtkWidget *dialog;
   GtkWidget *toplevel;
 
@@ -217,7 +216,6 @@ photos_selection_toolbar_properties_clicked (GtkButton *button, gpointer user_da
   GList *windows;
   GtkApplication *app;
   GtkWidget *dialog;
-  GtkWidget *toplevel;
   const gchar *urn;
 
   app = photos_application_new ();
@@ -517,7 +515,8 @@ photos_selection_toolbar_init (PhotosSelectionToolbar *self)
   image = gtk_image_new_from_icon_name ("list-add-symbolic", GTK_ICON_SIZE_INVALID);
   gtk_image_set_pixel_size (GTK_IMAGE (image), 16);
   gtk_container_add (GTK_CONTAINER (priv->toolbar_collection), image);
-  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->toolbar_collection), _("Organize"));
+  /* Translators: "Organize" refers to photos in this context */
+  gtk_widget_set_tooltip_text (GTK_WIDGET (priv->toolbar_collection), C_("Toolbar button tooltip", "Organize"));
   gtk_container_add (GTK_CONTAINER (priv->right_box), priv->toolbar_collection);
   g_signal_connect (priv->toolbar_collection,
                     "clicked",
