@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -48,6 +48,9 @@ photos_source_manager_client_account_added (GoaClient *client, GoaObject *object
   PhotosSourceManager *self = PHOTOS_SOURCE_MANAGER (user_data);
   PhotosSource *source;
 
+  if (goa_object_peek_photos (object) == NULL)
+    return;
+
   source = photos_source_new_from_goa_object (object);
   photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (source));
   g_object_unref (source);
@@ -70,9 +73,10 @@ photos_source_manager_client_account_removed (GoaClient *client, GoaObject *obje
 static void
 photos_source_manager_client_account_changed (GoaClient *client, GoaObject *object, gpointer user_data)
 {
-  PhotosSourceManager *self = PHOTOS_SOURCE_MANAGER (user_data);
-  photos_source_manager_client_account_removed (client, object, user_data);
-  photos_source_manager_client_account_added (client, object, user_data);
+  if (goa_object_peek_photos (object) == NULL)
+    photos_source_manager_client_account_removed (client, object, user_data);
+  else
+    photos_source_manager_client_account_added (client, object, user_data);
 }
 
 
@@ -86,15 +90,18 @@ photos_source_manager_refresh_accounts (PhotosSourceManager *self)
   accounts = goa_client_get_accounts (priv->client);
   for (l = accounts; l != NULL; l = l->next)
     {
+      GoaObject *object = GOA_OBJECT (l->data);
       PhotosSource *source;
 
-      if (goa_object_peek_account (GOA_OBJECT (l->data)) == NULL)
+      if (goa_object_peek_account (object) == NULL)
         continue;
 
-      /* TODO: uncomment when we start supporting online providers */
-      /* source = photos_source_new_from_goa_object (GOA_OBJECT (l->data)); */
-      /* photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (source)); */
-      /* g_object_unref (source); */
+      if (goa_object_peek_photos (object) == NULL)
+        continue;
+
+      source = photos_source_new_from_goa_object (GOA_OBJECT (l->data));
+      photos_base_manager_add_object (PHOTOS_BASE_MANAGER (self), G_OBJECT (source));
+      g_object_unref (source);
     }
 
   g_list_free_full (accounts, g_object_unref);
@@ -188,3 +195,46 @@ photos_source_manager_new (void)
 {
   return g_object_new (PHOTOS_TYPE_SOURCE_MANAGER, NULL);
 }
+
+
+GList *
+photos_source_manager_get_for_provider_type (PhotosSourceManager *self, const gchar *provider_type)
+{
+  GHashTable *sources;
+  GHashTableIter iter;
+  GList *items = NULL;
+  PhotosSource *source;
+
+  sources = photos_base_manager_get_objects (PHOTOS_BASE_MANAGER (self));
+  g_hash_table_iter_init (&iter, sources);
+  while (g_hash_table_iter_next (&iter, NULL, (gpointer) &source))
+    {
+      GoaAccount *account;
+      GoaObject *object;
+
+      object = photos_source_get_goa_object (source);
+      if (object == NULL)
+        continue;
+
+      account = goa_object_peek_account (object);
+      if (g_strcmp0 (goa_account_get_provider_type (account), provider_type) == 0)
+        items = g_list_prepend (items, g_object_ref (source));
+    }
+
+  return items;
+}
+
+
+gboolean
+photos_source_manager_has_provider_type (PhotosSourceManager *self, const gchar *provider_type)
+{
+  GList *items;
+  gboolean ret_val = FALSE;
+
+  items = photos_source_manager_get_for_provider_type (self, provider_type);
+  if (items != NULL)
+    ret_val = TRUE;
+
+  g_list_free_full (items, g_object_unref);
+  return ret_val;
+}
