@@ -1,6 +1,6 @@
 /*
  * Photos - access, organize and share your photos on GNOME
- * Copyright © 2012 Red Hat, Inc.
+ * Copyright © 2012, 2013 Red Hat, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,7 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
+#include "photos-collection-manager.h"
 #include "photos-item-manager.h"
 #include "photos-marshalers.h"
 #include "photos-query-builder.h"
@@ -39,6 +40,7 @@
 struct _PhotosTrackerControllerPrivate
 {
   GCancellable *cancellable;
+  PhotosBaseManager *col_mngr;
   PhotosBaseManager *item_mngr;
   PhotosBaseManager *src_mngr;
   PhotosOffsetController *offset_cntrlr;
@@ -120,7 +122,6 @@ photos_tracker_controller_cursor_next (GObject *source_object, GAsyncResult *res
     {
       tracker_sparql_cursor_close (cursor);
       photos_tracker_controller_query_finished (self, NULL);
-      g_object_unref (cursor);
       g_object_unref (self);
       return;
     }
@@ -157,6 +158,7 @@ photos_tracker_controller_query_executed (GObject *source_object, GAsyncResult *
                                     self->priv->cancellable,
                                     photos_tracker_controller_cursor_next,
                                     g_object_ref (self));
+  g_object_unref (cursor);
 }
 
 
@@ -168,7 +170,7 @@ photos_tracker_controller_perform_current_query (PhotosTrackerController *self)
   if (priv->current_query != NULL)
     photos_query_free (priv->current_query);
 
-  priv->current_query = PHOTOS_TRACKER_CONTROLLER_GET_CLASS (self)->get_query ();
+  priv->current_query = PHOTOS_TRACKER_CONTROLLER_GET_CLASS (self)->get_query (self);
   g_cancellable_reset (priv->cancellable);
 
   photos_tracker_queue_select (priv->queue,
@@ -191,6 +193,8 @@ photos_tracker_controller_offset_changed (PhotosOffsetController *offset_cntrlr,
 static void
 photos_tracker_controller_refresh_for_object (PhotosBaseManager *manager, GObject *object, gpointer user_data)
 {
+  PhotosTrackerController *self = PHOTOS_TRACKER_CONTROLLER (user_data);
+  photos_tracker_controller_refresh_internal (self, PHOTOS_TRACKER_REFRESH_FLAGS_RESET_OFFSET);
 }
 
 
@@ -275,6 +279,7 @@ photos_tracker_controller_dispose (GObject *object)
   PhotosTrackerController *self = PHOTOS_TRACKER_CONTROLLER (object);
   PhotosTrackerControllerPrivate *priv = self->priv;
 
+  g_clear_object (&priv->col_mngr);
   g_clear_object (&priv->item_mngr);
   g_clear_object (&priv->src_mngr);
   g_clear_object (&priv->offset_cntrlr);
@@ -308,6 +313,12 @@ photos_tracker_controller_init (PhotosTrackerController *self)
   priv->cancellable = g_cancellable_new ();
   priv->item_mngr = photos_item_manager_new ();
 
+  priv->col_mngr = photos_collection_manager_new ();
+  g_signal_connect (priv->col_mngr,
+                    "active-changed",
+                    G_CALLBACK (photos_tracker_controller_refresh_for_object),
+                    self);
+
   priv->src_mngr = photos_source_manager_new ();
   g_signal_connect (priv->src_mngr,
                     "object-added",
@@ -342,7 +353,7 @@ photos_tracker_controller_class_init (PhotosTrackerControllerClass *class)
                                                         query_error),
                                        NULL, /*accumulator */
                                        NULL, /*accu_data */
-                                       _photos_marshal_VOID__CHAR_CHAR,
+                                       _photos_marshal_VOID__STRING_STRING,
                                        G_TYPE_NONE,
                                        2,
                                        G_TYPE_STRING,
