@@ -30,6 +30,7 @@
 #include "photos-enums.h"
 #include "photos-item-manager.h"
 #include "photos-load-more-button.h"
+#include "photos-remote-display-manager.h"
 #include "photos-selection-controller.h"
 #include "photos-tracker-collections-controller.h"
 #include "photos-tracker-favorites-controller.h"
@@ -46,13 +47,11 @@ struct _PhotosViewContainerPrivate
   GtkWidget *load_more;
   PhotosBaseManager *item_mngr;
   PhotosModeController *mode_cntrlr;
+  PhotosRemoteDisplayManager *remote_mngr;
   PhotosSelectionController *sel_cntrlr;
   PhotosTrackerController *trk_cntrlr;
   PhotosWindowMode mode;
   gboolean disposed;
-  gulong adjustment_changed_id;
-  gulong adjustment_value_id;
-  gulong scrollbar_visible_id;
 };
 
 enum
@@ -107,20 +106,23 @@ photos_view_container_connect_view (PhotosViewContainer *self)
   GtkWidget *vscrollbar;
 
   vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->view));
-  priv->adjustment_changed_id = g_signal_connect_swapped (vadjustment,
-                                                          "changed",
-                                                          G_CALLBACK (photos_view_container_view_changed),
-                                                          self);
-  priv->adjustment_value_id = g_signal_connect_swapped (vadjustment,
-                                                        "value-changed",
-                                                        G_CALLBACK (photos_view_container_view_changed),
-                                                        self);
+  g_signal_connect_object (vadjustment,
+                           "changed",
+                           G_CALLBACK (photos_view_container_view_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
+  g_signal_connect_object (vadjustment,
+                           "value-changed",
+                           G_CALLBACK (photos_view_container_view_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   vscrollbar = gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (priv->view));
-  priv->scrollbar_visible_id = g_signal_connect_swapped (vscrollbar,
-                                                         "notify::visible",
-                                                         G_CALLBACK (photos_view_container_view_changed),
-                                                         self);
+  g_signal_connect_object (vscrollbar,
+                           "notify::visible",
+                           G_CALLBACK (photos_view_container_view_changed),
+                           self,
+                           G_CONNECT_SWAPPED);
 
   photos_view_container_view_changed (self);
 }
@@ -136,23 +138,8 @@ photos_view_container_disconnect_view (PhotosViewContainer *self)
   vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (priv->view));
   vscrollbar = gtk_scrolled_window_get_vscrollbar (GTK_SCROLLED_WINDOW (priv->view));
 
-  if (priv->adjustment_changed_id != 0)
-    {
-      g_signal_handler_disconnect (vadjustment, priv->adjustment_changed_id);
-      priv->adjustment_changed_id = 0;
-    }
-
-  if (priv->adjustment_value_id != 0)
-    {
-      g_signal_handler_disconnect (vadjustment, priv->adjustment_value_id);
-      priv->adjustment_value_id = 0;
-    }
-
-  if (priv->scrollbar_visible_id != 0)
-    {
-      g_signal_handler_disconnect (vscrollbar, priv->scrollbar_visible_id);
-      priv->scrollbar_visible_id = 0;
-    }
+  g_signal_handlers_disconnect_by_func (vadjustment, photos_view_container_view_changed, self);
+  g_signal_handlers_disconnect_by_func (vscrollbar, photos_view_container_view_changed, self);
 }
 
 
@@ -163,7 +150,16 @@ photos_view_container_item_activated (GdMainView *main_view,
                                       gpointer user_data)
 {
   PhotosViewContainer *self = PHOTOS_VIEW_CONTAINER (user_data);
-  photos_base_manager_set_active_object_by_id (self->priv->item_mngr, id);
+  PhotosViewContainerPrivate *priv = self->priv;
+  GObject *object;
+
+  object = photos_base_manager_get_object_by_id (priv->item_mngr, id);
+
+  if (!photos_base_item_is_collection (PHOTOS_BASE_ITEM (object)) &&
+      photos_remote_display_manager_is_active (priv->remote_mngr))
+    photos_remote_display_manager_render (priv->remote_mngr, PHOTOS_BASE_ITEM (object));
+  else
+    photos_base_manager_set_active_object (priv->item_mngr, object);
 }
 
 
@@ -291,6 +287,8 @@ photos_view_container_constructed (GObject *object)
                     G_CALLBACK (photos_view_container_window_mode_changed),
                     self);
 
+  priv->remote_mngr = photos_remote_display_manager_dup_singleton ();
+
   switch (priv->mode)
     {
     case PHOTOS_WINDOW_MODE_COLLECTIONS:
@@ -345,6 +343,8 @@ photos_view_container_dispose (GObject *object)
 
   g_clear_object (&priv->model);
   g_clear_object (&priv->item_mngr);
+  g_clear_object (&priv->mode_cntrlr);
+  g_clear_object (&priv->remote_mngr);
   g_clear_object (&priv->sel_cntrlr);
   g_clear_object (&priv->trk_cntrlr);
 
@@ -373,8 +373,6 @@ photos_view_container_set_property (GObject *object, guint prop_id, const GValue
 static void
 photos_view_container_init (PhotosViewContainer *self)
 {
-  PhotosViewContainerPrivate *priv;
-
   self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
                                             PHOTOS_TYPE_VIEW_CONTAINER,
                                             PhotosViewContainerPrivate);
